import 'dart:io';

import 'package:flutter_test/flutter_test.dart';
import 'package:lokalise_flutter_sdk/src/extensions/directory.dart';
import 'package:lokalise_flutter_sdk/src/generator/dart_generator/synthetic_package_generator.dart';

import '../../../assets/assets_routes.dart';

void main() {
  group('SyntheticPackageGenerator', () {
    late Directory directory;
    late SyntheticPackageGenerator generator;

    setUp(() async {
      directory =
          await Directory('$kGeneratedRoute/synthetic_package_generator')
              .create(recursive: true);
      generator = SyntheticPackageGenerator(outputDir: directory);
    });

    tearDown(() {
      directory.deleteSync(recursive: true);
    });

    test('pubspec.yaml already exists', () async {
      final file = await directory.createFile(withName: 'pubspec.yaml');
      const content = 'hello: hello\n';

      file.writeAsStringSync(content);

      await generator.generate();

      expect(file.readAsStringSync(), equals(content));
      expect(directory.listSync().length, equals(1));
    });

    test('pubspec.yaml not exists', () async {
      await generator.generate();

      final file = directory.getFile(withName: 'pubspec.yaml');
      expect(file, isNotNull);
      expect(file!.readAsStringSync(), equals(_expectedFileContent));
    });
  });
}

String get _expectedFileContent => '''
# Generated by the flutter tool
name: synthetic_package
description: The Flutter application's synthetic package.
''';
